import { BaseEvent } from '#application/base/baseEvent'
import { ZoneEventTileType } from '#application/enums'
import { UUID } from '#application/types'
import { Zone } from '#entities/zone'
import { ZoneEffect } from '#entities/zoneEffect'
import { ZoneEventTile } from '#entities/zoneEventTile'
import { ZoneEventTileTeleport } from '#entities/zoneEventTileTeleport'
import { ZoneObject } from '#entities/zoneObject'
import zoneManager from '#managers/zoneManager'
import ZoneRepository from '#repositories/zoneRepository'

interface IPayload {
  zoneId: UUID
  name: string
  width: number
  height: number
  tiles: string[][]
  pvp: boolean
  zoneEventTiles: {
    type: ZoneEventTileType
    positionX: number
    positionY: number
    teleport?: {
      toZoneId: UUID
      toPositionX: number
      toPositionY: number
      toRotation: number
    }
  }[]
  zoneEffects: {
    effect: string
    strength: number
  }[]
  zoneObjects: ZoneObject[]
}

export default class ZoneUpdateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:zone_editor:zone:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: Zone | null) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const character = await this.getCharacter()
      this.logger.info(`User ${character!.getId()} has updated zone via zone editor.`)

      if (!data.zoneId) {
        this.logger.info(`User ${character!.getId()} tried to update zone but did not provide a zone id.`)
        return callback(null)
      }

      let zone = await ZoneRepository.getById(data.zoneId)

      if (!zone) {
        this.logger.info(`User ${character!.getId()} tried to update zone ${data.zoneId} but it does not exist.`)
        return callback(null)
      }

      // Validation logic remains the same
      if (data.tiles.length > data.height) {
        data.tiles = data.tiles.slice(0, data.height)
      }
      for (let i = 0; i < data.tiles.length; i++) {
        if (data.tiles[i].length > data.width) {
          data.tiles[i] = data.tiles[i].slice(0, data.width)
        }
      }

      data.zoneEventTiles = data.zoneEventTiles.filter((tile) => tile.positionX >= 0 && tile.positionX < data.width && tile.positionY >= 0 && tile.positionY < data.height)

      data.zoneObjects = data.zoneObjects.filter((obj) => obj.positionX >= 0 && obj.positionX < data.width && obj.positionY >= 0 && obj.positionY < data.height)

      // Clear existing collections
      zone.zoneEventTiles.removeAll()
      zone.zoneObjects.removeAll()
      zone.zoneEffects.removeAll()

      // Create and add new zone event tiles
      for (const tile of data.zoneEventTiles) {
        const zoneEventTile = new ZoneEventTile().setType(tile.type).setPositionX(tile.positionX).setPositionY(tile.positionY).setZone(zone)

        if (tile.teleport) {
          const teleport = new ZoneEventTileTeleport()
            .setToZone((await ZoneRepository.getById(tile.teleport.toZoneId))!)
            .setToPositionX(tile.teleport.toPositionX)
            .setToPositionY(tile.teleport.toPositionY)
            .setToRotation(tile.teleport.toRotation)

          zoneEventTile.setTeleport(teleport)
        }

        zone.zoneEventTiles.add(zoneEventTile)
      }

      // Create and add new zone objects
      for (const object of data.zoneObjects) {
        const zoneObject = new ZoneObject().setMapObject(object.mapObject).setDepth(object.depth).setIsRotated(object.isRotated).setPositionX(object.positionX).setPositionY(object.positionY).setZone(zone)

        zone.zoneObjects.add(zoneObject)
      }

      // Create and add new zone effects
      for (const effect of data.zoneEffects) {
        const zoneEffect = new ZoneEffect().setEffect(effect.effect).setStrength(effect.strength).setZone(zone)

        zone.zoneEffects.add(zoneEffect)
      }

      // Update zone properties
      await zone.setName(data.name).setWidth(data.width).setHeight(data.height).setTiles(data.tiles).setPvp(data.pvp).setUpdatedAt(new Date()).update()

      // Reload zone from database to get fresh data
      zone = await ZoneRepository.getById(data.zoneId)

      if (!zone) {
        this.logger.info(`User ${character!.getId()} tried to update zone ${data.zoneId} but it does not exist after update.`)
        return callback(null)
      }

      // Reload zone for players
      zoneManager.unloadZone(data.zoneId)
      await zoneManager.loadZone(zone)

      return callback(zone)
    } catch (error: any) {
      this.logger.error(`gm:zone_editor:zone:update error: ${error instanceof Error ? error.message : String(error)}`)
      return callback(null)
    }
  }
}
